/*****************************************************************************
 * Copyright (c) 2018 CEA.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Benoit Maggi (CEA LIST) benoit.maggi@cea.fr - Initial API and implementation
 *****************************************************************************/

package org.eclipse.papyrus.iotml;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.papyrus.sysml14.util.SysMLResource;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Model;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.Profile;
import org.eclipse.uml2.uml.UMLFactory;
import org.eclipse.uml2.uml.UMLPackage;
import org.eclipse.uml2.uml.resource.UMLResource;

/**
 * Utility class to get informations on IoT resources
 */
public final class IoTMLResource {

	@Deprecated // FIXME should be defined in Marte plugin
	public static final String MARTE_PROFILE_PATH = "platform:/plugin/org.eclipse.papyrus.marte.static.profile/resources/MARTE.profile.uml";

	public static final String PROFILES_PATHMAP = "pathmap://IOTML_PROFILE/"; //$NON-NLS-1$	
	
	public static final String PROFILE_PATH = PROFILES_PATHMAP+"IoT-ML.profile.uml"; //$NON-NLS-1$

	public static final String PROFILE_URI = "http://www.eclipse.org/papyrus/iotml/0.7/IoTML"; //$NON-NLS-1$

	private static final String UNNAMED = "unnamed"; //$NON-NLS-1$

	private IoTMLResource(){
	}
	
	/**
	 * Creates a model with IoTML profile applied
	 *
	 * @param resourceSet
	 *        the resource set in which to create the model
	 * @return the created model
	 */
	public static Model createIoTMLModel(ResourceSet resourceSet) {//, String modelName
		return createIoTMLModel(resourceSet, UNNAMED+"."+UMLResource.FILE_EXTENSION, UNNAMED); //$NON-NLS-1$
	}

	/**
	 * Creates a IoTML model with SysML, Marte and IoTML profiles applied
	 *  
	 * @param resourceSet the resource set in which to create the model
	 * @param resourceName name of the resource
	 * @param modelName name of the model
	 * @return the created model
	 */
	public static Model createIoTMLModel(ResourceSet resourceSet, String resourceName, String modelName) {
		Model model = UMLFactory.eINSTANCE.createModel();
		model.setName(modelName);
		Resource resource = resourceSet.createResource(URI.createURI(resourceName));
		resource.getContents().add(model);
		// Apply UML Standard profile
		// Retrieve standard profile
		Resource standardProfileResource = resourceSet.getResource(URI.createURI(UMLResource.STANDARD_PROFILE_URI), true);
		Profile umlStdProfile = (Profile) EcoreUtil.getObjectByType(standardProfileResource.getContents(), UMLPackage.Literals.PACKAGE);
		// Apply to the model
		model.applyProfile(umlStdProfile);
		
		// Retrieve and apply SysML profile
		Resource sysmlProfileResource = resourceSet.getResource(URI.createURI(SysMLResource.PROFILE_PATH), true);
		Profile sysmlProfile = (Profile) EcoreUtil.getObjectByType(sysmlProfileResource.getContents(), UMLPackage.Literals.PACKAGE);
		model.applyProfile(sysmlProfile);
		
		// Retrieve and apply Marte profile
		Resource marteProfileResource = resourceSet.getResource(URI.createURI(MARTE_PROFILE_PATH), true);
		Profile marteProfile = (Profile) EcoreUtil.getObjectByType(marteProfileResource.getContents(), UMLPackage.Literals.PACKAGE);
		model.applyProfile(marteProfile);
		
		// Retrieve and apply IoTML profile
		Resource iotMLProfileResource = resourceSet.getResource(URI.createURI(IoTMLResource.PROFILE_PATH), true);
		Profile iotProfile = (Profile) EcoreUtil.getObjectByType(iotMLProfileResource.getContents(), UMLPackage.Literals.PACKAGE);
		model.applyProfile(iotProfile);		
		
		return model;
	}

	/**
	 * Recursive function
	 * @param element the initial uml2 element
	 * @param profileLookup the profile we are looking for
	 * @return a package, a model profiled by IoTML or null
	 */
	public static Package getFirstMatchingContainerProfiledByIoTMLProfile(Element element){
		return getFirstMatchingContainerProfiledBy(element,IoTMLResource.PROFILE_URI);
	}

	/**
	 * Recursive function
	 * @param element the initial uml2 element
	 * @param profileUri the profile we are looking for
	 * @return a package, a model or null
	 */
	public static Package getFirstMatchingContainerProfiledBy(Element element, String profileUri){
		if(element instanceof Package){
			Package pakage = (Package)element;
			
			EList<Profile>  appliedProfiles = pakage.getAppliedProfiles();
			if (appliedProfiles!=null){
				for (Profile appliedProfile : appliedProfiles) {
					//as soon as we match with the profile we return true
					if(appliedProfile.getURI()!=null && appliedProfile.getURI().compareTo(profileUri)==0){
						return pakage;
					}
				}		
			}

			Element elementOwner = element.getOwner();
			if(elementOwner!=null){
				return getFirstMatchingContainerProfiledBy(elementOwner, profileUri);
			}
		}
		//If no match then look at the nearest package
		if(element!=null){
			Package myNearestPackage = element.getNearestPackage();
			if(myNearestPackage!=element){
				return getFirstMatchingContainerProfiledBy(myNearestPackage, profileUri);
			}
		}
		//If we are here, it means we have no lookuped profile attached at one container of our element.
		return null;
	}


	/**
	 * @param element
	 * @return true if the element is profiled with SysML profile
	 */
	public static boolean isIoTMLProfileApplied(Element element){
		return isProfiledByProfile(element,IoTMLResource.PROFILE_URI);
	}


	/**
	 * @param element
	 * @param profileUri
	 * @return true if the element is profiled by the given profileUri
	 */
	public static boolean isProfiledByProfile(Element element, String profileUri){
		return getFirstMatchingContainerProfiledBy(element,profileUri)!=null;
	}
}
